using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Application.Common.Mappings.Resolvers;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.TicketGroups;

public sealed class TicketGroupDto : IMapFrom<TicketGroup>
{
    public Guid Uuid { get; set; }

    public string PassangerFirstName { get; set; }

    public string PassangerLastName { get; set; }

    public string PassangerPatronymic { get; set; }

    public string PassangerSex { get; set; }

    public DateOnly PassangerBirthDate { get; set; }

    public string? PassangerEmail { get; set; }

    public DateTimeOffset PurchaseTime { get; set; }

    public string Status { get; set; }


    public DateTimeOffset DepartureTime { get; set; }

    public DateTimeOffset ArrivalTime { get; set; }

    public TimeSpan TravelTime { get; set; }

    public TimeSpan TimeInStops { get; set; }

    public int NumberOfTransfers { get; set; }

    public string Currency { get; set; }

    public decimal Cost { get; set; }

    public TicketGroupAccountDto? Account { get; set; }

    public ICollection<TicketGroupVehicleEnrollmentDto> Enrollments { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<TicketGroup, TicketGroupDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.PassangerSex,
                opt => opt.MapFrom(s => s.PassangerSex.Name))
            .ForMember(
                d => d.PurchaseTime,
                opt => opt
                    .MapFrom<DateTimeOffsetToLocalResolver, DateTimeOffset>(
                        s => s.PurchaseTime))
            .ForMember(
                d => d.Status,
                opt => opt.MapFrom(s => s.Status.Name))
            .ForMember(
                d => d.DepartureTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var departureRouteAddressId = 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First()
                                .DepartureRouteAddressId;
                        return 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First().VehicleEnrollment
                                .GetDepartureTime(departureRouteAddressId);
                    }))
            .ForMember(
                d => d.ArrivalTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var arrivalRouteAddressId = 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First()
                                .ArrivalRouteAddressId;
                        return 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First().VehicleEnrollment
                                .GetArrivalTime(arrivalRouteAddressId);
                    }))
            .ForMember(
                d => d.TravelTime,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var departureRouteAddressId = 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First()
                                .DepartureRouteAddressId;
                        var arrivalRouteAddressId = 
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First()
                                .ArrivalRouteAddressId;
                        var departureTime =
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First().VehicleEnrollment
                                .GetDepartureTime(departureRouteAddressId);
                        var arrivalTime =
                            s.Tickets
                                .OrderBy(t => t.Order)
                                .First().VehicleEnrollment
                                .GetArrivalTime(departureRouteAddressId);
                        return arrivalTime - departureTime;
                    }))
            .ForMember(
                d => d.TimeInStops,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var timePeriodsInStops = 
                            s.Tickets.Select(t => 
                                {
                                    var departureRouteAddressId =
                                        t.DepartureRouteAddressId;
                                    var arrivalRouteAddressId =
                                        t.ArrivalRouteAddressId;
                                    return
                                        t.VehicleEnrollment.GetTimeInStops(
                                            departureRouteAddressId,
                                            arrivalRouteAddressId);
                                });
                        return
                            timePeriodsInStops
                                .Aggregate(TimeSpan.Zero, 
                                    (sum, next) => sum += next);
                    }))
            .ForMember(
                d => d.NumberOfTransfers,
                opt => opt.MapFrom(s => s.Tickets.Count() - 1))
            .ForMember(
                d => d.Cost,
                opt => opt.MapFrom(
                    (s, d) =>
                    {
                        var costs = 
                            s.Tickets.Select(t => t.Currency.Round(t.Cost));
                        return
                            costs
                                .Aggregate((decimal)0, 
                                    (sum, next) => sum += next);
                    }))
            .ForMember(
                d => d.Enrollments,
                opt => opt.MapFrom(s => s.Tickets));
    }
}
